# Copyright 2019 Cumulus Networks, Inc.  All Rights reserved.
#
# This software is subject to the Cumulus Networks End User License Agreement
# available at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt

"""
General search

Usage:
    netq search <text-search-prefix> [json]

Options:
    search                      : Perform general search on a given prefix
    <text-search-prefix>        : Criteria to search
"""



from netq_lib.common.enums import (
    HeaderLength,
)
from netq_apps.cmd.netq import netq_tabulate as tabulate
from netq_apps.modules import NetqModule, RC_SUCCESS
from netq_lib.orm.redisdb.models import Search
app = NetqModule()


@app.route('search')
def cli_search(cli, netq):
    prefix = cli.get('<text-search-prefix>')
    jsonify = True if cli.get('json') else False
    col_headers = ['Hostname', 'Type', 'IP Address']
    col_sizes = [HeaderLength.get_col_size(header) for header in col_headers]
    output = []
    for entry in Search.query.request(path='match', prefix=prefix):
        output.append([entry.hostname, entry.type, entry.ip_address])
    if len(output):
        tabulate(output=output, headers=col_headers, colsizes=col_sizes, jsonify=jsonify)
    else:
        print(('\nNo matches for {} found'.format(prefix)))
    return RC_SUCCESS
