# Copyright 2017, 2018, 2019, 2020, 2021 Cumulus Networks, Inc.  All Rights reserved.
#
# This software is subject to the Cumulus Networks End User License Agreement
# available at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt
import os


class AGENTSTATUS(object):
    SUCCESS = 0
    FAILURE = 1


class HeaderLength(object):
    NTP_SYNC_LEN = 8
    STRATUM_LEN = 7
    NTP_APP_LEN = 21
    DB_STATE_LEN = 10
    TIME_COL_LEN = 25
    HOST_COL_LEN = 17
    VERSION_COL_LEN = 36
    STATUS_COL_LEN = 8
    IP_ADDR_COL_LEN = 25
    VRF_COL_LEN = 15
    PORT_COL_LEN = 10
    ERROR_COL_LEN = 15
    MAC_ADDR_COL_LEN = 18
    VLAN_COL_LEN = 25
    PROC_ID_LEN = 5
    SERVICE_LEN = 20
    DESC_LEN = 35
    MSG_LEN = 35
    REASON_LEN = 45
    STATE_LEN = 10
    OS_LEN = 15
    ARCH_LEN = 8
    MODEL_LEN = 30
    FREQ_LEN = 10
    SIZE_LEN = 10
    SERIAL_NO_LEN = 25
    VENDOR_LEN = 20
    SPEED_LEN = 10
    INTERFACE_LEN = 25
    CLASS_LEN = 30
    KEY_FIELDS_LEN = 100
    ROLE_LEN = 10
    REPLICAS_LEN = HOST_COL_LEN * 2
    KEYS_LEN = 7
    ID_LEN = 25
    SWITCH_LEN = 20
    ASIC_LEN = 15
    PREFIX_LEN = 30
    MODULE_LEN = 9
    PART_NO_LEN = 16
    TRANSCEIVER_LEN = 16
    CONNECTOR_LEN = 16
    LENGTH_LEN = 6
    AUTO_NEG_LEN = 7
    TEMP_LEN = CRITICAL_LEN = MAX_LEN = MIN_LEN = 8
    NEIGHBOR_LEN = 28
    ASN_LEN = 10
    PEER_ASN_LEN = 10
    PFXRX_LEN = 12
    VNI_LEN = 10
    VTEP_IP_LEN = 16
    IN_KERNEL_LEN = 9
    EXPORT_RT_LEN = IMPORT_RT_LEN = 16
    PEER_NODE_LEN = 10
    CONTAINER_NAME_LEN = 20
    CONTAINER_IP_LEN = 20
    NETWORK_NAME_LEN = 10
    SERVICE_NAME_LEN = 15
    SWARM_SERVICE_LEN = 9
    PEER_PORT_LEN = 20
    PROTO_LEN = 6
    HOST_IP_LEN = 21
    HOST_PORT_LEN = 12
    IP_MASQ_LEN = 8
    INSTALLED_LEN = RUNNING_LEN = IMAGES_LEN = 10
    SWARM_CLUSTERS_LEN = 6
    NETWORKS_LEN = 12
    DRIVER_LEN = 9
    SUBNET_LEN = GATEWAY_LEN = 15
    CONTAINERS_LEN = 25
    MANAGER_LEN = MODE_LEN = CLUSTER_LEN = 10
    CLUSTER_NAME_LEN = 15
    NUM_NODES_LEN = 9
    MANAGER_NODES_LEN = NODES_LEN = 40
    WORKER_NODES_LEN = 30
    TYPE_LEN = 16
    REMOTE_LEN = 6
    CONTROLLER_LEN = 20
    SCHEDULER_LEN = 12
    NODE_NAME_LEN = 20
    KUBELET_LEN = KUBEPROXY_LEN = 12
    CONTAINER_RUNTIME_LEN = 16
    LABELS_LEN = 20
    POD_CIDR_LEN = 24
    NAMESPACE_LEN = 16
    DAEMON_SET_NAME_LEN = 20
    DESIRED_COUNT_LEN = READY_COUNT_LEN = READY_REPLICAS_LEN = 8
    NODE_LEN = 12
    REPLICATION_NAME_LEN = 20
    CLUSTER_IP_LEN = EXTERNAL_IP_LEN = 16
    BACKUP_LEN = 6
    NUM_BONDS_LEN = NUM_DUAL_LEN = 5
    VLAN_LEN = 6
    SVI_LEN = 4
    AREA_LEN = 12
    REPL_MODE_LEN = 8
    NUM_PEERS_LEN = 6
    VNIS_LEN = 6
    MTU_LEN = 6
    DATE_LEN = 10
    TABLE_LEN = 15
    DISCOVERY_STATUS_LEN = 16
    UPGRADE_STATUS_LEN = 14

    # If a reasonable header length cannot be determined,
    # use the length of the header in some multiple of 2
    header_length_dict = {'Hostname': HOST_COL_LEN,
                          'NTP Sync': NTP_SYNC_LEN,
                          'Stratum': STRATUM_LEN,
                          'NTP App': NTP_APP_LEN,
                          'DB State': DB_STATE_LEN,
                          'DBState': DB_STATE_LEN,
                          'Peer Hostname': HOST_COL_LEN,
                          'Peer Name': HOST_COL_LEN,
                          'Peer': HOST_COL_LEN,
                          'Server': HOST_COL_LEN,
                          'Master': HOST_COL_LEN * 2,
                          'Current Server': HOST_COL_LEN,
                          'Last Changed': TIME_COL_LEN,
                          'Uptime': TIME_COL_LEN,
                          'Sys Uptime': TIME_COL_LEN,
                          'Agent Uptime': TIME_COL_LEN,
                          'Reinitialize Time': TIME_COL_LEN + 1,
                          'Version': VERSION_COL_LEN,
                          'Status': STATUS_COL_LEN * 2,
                          'Address': IP_ADDR_COL_LEN,
                          'IP Address': IP_ADDR_COL_LEN,
                          'IP': IP_ADDR_COL_LEN,
                          'SysMac': MAC_ADDR_COL_LEN,
                          'Base MAC': MAC_ADDR_COL_LEN,
                          'MAC Address': MAC_ADDR_COL_LEN,
                          'VRF': VRF_COL_LEN,
                          'Port': PORT_COL_LEN,
                          'Error': ERROR_COL_LEN,
                          'VLANs': VLAN_COL_LEN,
                          'SVIs': VLAN_COL_LEN,
                          'PID': PROC_ID_LEN,
                          'Service': SERVICE_LEN,
                          'Description': DESC_LEN,
                          's_desc': DESC_LEN,
                          'Details': DESC_LEN,
                          'Message': MSG_LEN,
                          'Ports': MSG_LEN,
                          'Reason': REASON_LEN,
                          'State': STATE_LEN,
                          'OS': OS_LEN,
                          'Name': OS_LEN,
                          's_name': OS_LEN,
                          'CPU': ARCH_LEN,
                          'Arch': ARCH_LEN,
                          'Model': MODEL_LEN,
                          'Freq': FREQ_LEN,
                          'Size': SIZE_LEN,
                          'Memory': SIZE_LEN,
                          'Serial No': SERIAL_NO_LEN,
                          'Vendor': VENDOR_LEN,
                          'Speed': SPEED_LEN,
                          'Interface': INTERFACE_LEN,
                          'Peer Interface': INTERFACE_LEN,
                          'Class': CLASS_LEN,
                          'Key Fields': KEY_FIELDS_LEN,
                          'Role': ROLE_LEN,
                          'Replicas': REPLICAS_LEN,
                          'Keys': KEYS_LEN,
                          'Id': ID_LEN,
                          'Switch': SWITCH_LEN,
                          'ASIC': ASIC_LEN,
                          'Prefix': PREFIX_LEN,
                          'Module': MODULE_LEN,
                          'Part No': PART_NO_LEN,
                          'Transceiver': TRANSCEIVER_LEN,
                          'Connector': CONNECTOR_LEN,
                          'Length': LENGTH_LEN,
                          'Auto Neg': AUTO_NEG_LEN,
                          'AutoNeg': AUTO_NEG_LEN,
                          'Temp': TEMP_LEN,
                          'Critical': CRITICAL_LEN,
                          'Max': MAX_LEN,
                          'Min': MIN_LEN,
                          'Neighbor': NEIGHBOR_LEN,
                          'ASN': ASN_LEN,
                          'Peer ASN': PEER_ASN_LEN,
                          'PfxRx': PFXRX_LEN,
                          'VNI': VNI_LEN,
                          'VTEP IP': VTEP_IP_LEN,
                          'In Kernel': IN_KERNEL_LEN,
                          'Export RT': EXPORT_RT_LEN,
                          'Import RT': IMPORT_RT_LEN,
                          'Peer Node': PEER_NODE_LEN,
                          'Container Name': CONTAINER_NAME_LEN,
                          'Container IP': CONTAINER_IP_LEN,
                          'Network Name': NETWORK_NAME_LEN,
                          'Service Name': SERVICE_NAME_LEN,
                          'Swarm Service': SWARM_SERVICE_LEN,
                          'Peer Port': PEER_PORT_LEN,
                          'Proto': PROTO_LEN,
                          'Host IP': HOST_IP_LEN,
                          'Host Port': HOST_PORT_LEN,
                          'IP Masq': IP_MASQ_LEN,
                          'Installed': INSTALLED_LEN,
                          'Running': RUNNING_LEN,
                          'Images': IMAGES_LEN,
                          'Swarm Clusters': SWARM_CLUSTERS_LEN,
                          'Networks': NETWORKS_LEN,
                          'Driver': DRIVER_LEN,
                          'Subnet': SUBNET_LEN,
                          'Gateway': GATEWAY_LEN,
                          'Containers': CONTAINERS_LEN,
                          'Manager': MANAGER_LEN,
                          'Manager Node': MANAGER_LEN,
                          'Mode': MODE_LEN,
                          'Cluster': CLUSTER_LEN,
                          'Cluster Name': CLUSTER_NAME_LEN,
                          'Docker Version': len('Docker Version'),
                          'Availability': len('Availability'),
                          'Swarm Node': len('Swarm Node'),
                          'Node ID': len('Node ID'),
                          'Port Mapping': len('Port Mapping'),
                          'Virtual IP': len('Virtual IP'),
                          'Num Nodes': NUM_NODES_LEN,
                          'Worker Nodes': WORKER_NODES_LEN,
                          'Manager Nodes': MANAGER_NODES_LEN,
                          'Type': TYPE_LEN,
                          'Remote': REMOTE_LEN,
                          'Controller Status': CONTROLLER_LEN,
                          'Scheduler Status': SCHEDULER_LEN,
                          'Nodes': NODES_LEN,
                          'Node': NODE_LEN,
                          'Node Name': NODE_NAME_LEN,
                          'Kubelet': KUBELET_LEN,
                          'KubeProxy': KUBEPROXY_LEN,
                          'Container Runtime': CONTAINER_RUNTIME_LEN,
                          'Labels': LABELS_LEN,
                          'Pod CIDR': POD_CIDR_LEN,
                          'Namespace': NAMESPACE_LEN,
                          'Daemon Set Name': DAEMON_SET_NAME_LEN,
                          'Desired Count': DESIRED_COUNT_LEN,
                          'Ready Count': READY_COUNT_LEN,
                          'Ready Replicas': READY_REPLICAS_LEN,
                          'Replication Name': REPLICATION_NAME_LEN,
                          'Cluster IP': CLUSTER_IP_LEN,
                          'External IP': EXTERNAL_IP_LEN,
                          'Backup': BACKUP_LEN,
                          '#Bonds': NUM_BONDS_LEN,
                          '#Dual': NUM_DUAL_LEN,
                          'VLAN': VLAN_LEN,
                          'SVI': SVI_LEN,
                          'Area': AREA_LEN,
                          'ReplMode': REPL_MODE_LEN,
                          '#Peers': NUM_PEERS_LEN,
                          '#VNIs': VNIS_LEN,
                          'MTU': MTU_LEN,
                          'Peer MTU': len('Peer MTU'),
                          'Enabled': len('Enabled'),
                          'Active': len('Active'),
                          'Monitored': len('Monitored'),
                          'Rev': len('Rev') * 2,
                          'Mfg Date': DATE_LEN,
                          'Cores': len('Cores'),
                          'Model ID': ID_LEN,
                          'Core BW': len('Core BW') * 2,
                          'Peer ID': ID_LEN,
                          'PeerID': ID_LEN,
                          'Peer IP': IP_ADDR_COL_LEN,
                          'Origin': len('Origin'),
                          'Nexthops': MSG_LEN,
                          'Up Time': TIME_COL_LEN,
                          'Table': TABLE_LEN,
                          'Value Fields': KEY_FIELDS_LEN,
                          'Egress Port': PORT_COL_LEN * 2,
                          'Protocol': PROTO_LEN * 2,
                          'Peer VLANs': VLAN_COL_LEN,
                          'Replication List': MSG_LEN,
                          'Connect Time': TIME_COL_LEN,
                          'Value': len('Value') * 2,
                          'Sentinels': REPLICAS_LEN,
                          'Sentinel': len('Sentinel'),
                          'Database': len('Database') * 2,
                          'Transport': len('Transport') * 2,
                          'Timestamp': TIME_COL_LEN,
                          'Discovery Status': DISCOVERY_STATUS_LEN,
                          'Upgrade Status': UPGRADE_STATUS_LEN,
                          'WJH': len('Disable')+2,
                          'CPU Limit': len('CPU Limit'),
                          'Log Level': len('Log Level'),
                          'CL Version': len('CL Version')+1,
                          'NetQ Version': len('NetQ Version')+1,
                          'Prev NetQ Version': len('NetQ Version')+1,
                          'Overall Status': STATUS_COL_LEN*2,
                          'Pre-Check Status': STATUS_COL_LEN*2,
                          'Start Time': TIME_COL_LEN + 1,
                          }

    @classmethod
    def get_col_size(cls, header):
        # preserving case sensitivity in header_length_dict makes reading
        # acronyms much easier
        lower_dict = dict()
        for key in list(cls.header_length_dict.keys()):
            lower_dict[key.lower()] = cls.header_length_dict[key]

        return lower_dict.get(header.lower(), len(header) * 2)


class BgColors(object):
    """ Background Colors
    """
    HEADER = '\033[95m'
    BLUE = '\033[94m'
    YELLOW = '\033[93m'
    GREEN = '\033[92m'
    WARNING = '\033[93m'
    RED = '\033[91m'
    GRAY = '\033[90m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'
    COLORS = (BLUE, BOLD, GRAY, GREEN, RED, YELLOW)

    def __init__(self):
        raise NotImplementedError


class Daemons(object):
    """ Daemons running on the system
    """
    BGPD = 'bgpd'
    CLAG = 'clagd'
    LLDP = 'lldpd'
    MISC = 'misc'
    MSTPD = 'mstpd'
    OSPFD = 'ospfd'
    OSPF6D = 'ospf6d'
    ZEBRA = 'zebra'

    def __init__(self):
        raise NotImplementedError


class DefaultPaths(object):

    OS_RELEASE = '/etc/os-release'
    OPTA_FILE = '/etc/app-release'
    CONF_FILE = '/etc/netq/netq.yml'
    CONF_DIR = '/etc/netq/config.d/'
    RUN_DIR = '/var/run/netq/'
    VAR_LIB_DIR = '/var/lib/netq/'
    DBCONFIG = '%snetq-agent-commands.yml' % CONF_DIR
    DBCONFIG_RUNNING = '/var/run/netq-agent-running.json'
    INTERFACE_FILE = '/etc/network/interfaces'
    UDS_FILE = '/var/run/netq-agent.sock'
    INSTALLER_LOCK = '/var/run/netqd/opta-installer-lock'
    DEFAULT_COMMANDS = '/etc/netq/commands/'
    WORKING_COMMANDS = os.path.join(RUN_DIR, 'commands/')
    WORKING_CMD_FILE = os.path.join(RUN_DIR, 'netq_commands.yml')
    ADMIN_APP_CONF_FILE = '/mnt/admin/netq_admin_app.yml'
    ROCE_STATUS_FILE_PATH = '/etc/cumulus/switchd.d/qos.conf'
    ROCE_DUMMY_DATA_PATH = '/etc/netq/.test_roce.yml'
    TRAFFIC_CONF_FILE_PATH = '/etc/cumulus/datapath/traffic.conf'
    QOS_CONF_FILE_PATH = '/etc/cumulus/datapath/qos/qos_features.conf'

    def __init__(self):
        raise NotImplementedError


class CommandParams(object):

    SERVICE = 'service'
    SERVICEKEY = 'key'
    PERIOD = 'period'
    COMMAND = 'command'
    NETQCOMANDS = 'netq-commands'
    COMMANDS = 'commands'
    ISACTIVE = 'isactive'
    PARSER = 'parser'
    LOCALPARSER = 'local'
    EXTERNALPARSER = 'external'

    def __init__(self):
        raise  NotImplementedError


class PlatformType(object):

    OPTA = 'opta'
    ARMEL = 'armel'
    DEFAULT = 'default'
    CL3 = 'cl3'
    CL4 = 'cl4'
    SONIC = 'sonic'

    def __init__(self):
        raise NotImplementedError


class ConfKw(object):
    '''Keywords for the fields in user config'''
    HIDE_VALUES = 'hide-values'
    BACKEND = 'backend'
    NETQ_CLI = 'netq-cli'
    OPTA_CHECK = 'opta-check'
    SERVER = 'server'
    CLUSTER_SERVERS = 'cluster-servers'
    PORT = 'port'
    OPID = 'opid'
    ENDPOINT = 'endpoint'
    IP = 'ip'
    GATEWAY = 'gateway'
    REDIS = 'redis'
    VERSION = 'version'
    VRF = 'vrf'
    PREMISES = 'premises'
    NETQ_USER = 'netq-user'
    NETQD = 'netqd'
    NETQ_AGENT = 'netq-agent'
    NETQ_GNMI = 'netq-gnmi'
    HA = 'ha'
    ROLE = 'role'
    MASTER = 'master'
    SENTINEL = 'sentinel'
    REPLICA = 'replica'
    ENABLE_OPTA_DISCOVERY = 'enable-opta-discovery'
    IS_OPTA_ENABLED = 'is-opta-enabled'
    IS_GNMI_ENABLED = 'is-gnmi-enabled'
    AGENT_PORT = 'agentport'
    AGENT_URL = 'agenturl'
    IS_REDIS_ENABLED = 'is-redis-enabled'
    EXHIBIT_URL = 'exhibiturl'
    EXHIBIT_PORT = 'exhibitport'
    COUNT = 'count'
    MAX_BATCH_SIZE = 5000
    RANDOM_PLATFORM = 'random_platform'
    API_LOGGING = 'api-logging'
    FULLSTATE_PERIOD = 'fullstate-period'
    IS_LIBNL_ENABLED = 'is-libnl-enabled'
    IS_WJH_ENABLED = 'wjh'
    CPU_LIMIT = 'cpu-limit'
    OPTA_DISCOVERY_SERVERS = 'opta-discovery-servers'
    NETQ_STREAM_PORT = "netq_stream_port"
    NETQ_STREAM_ADDRESS = "netq_stream_address"

    HEALTH_PORT = 8443
    DEFAULT_API_LOGGING = False
    DEFAULT_BATCH_SIZE = 2000
    DEFAULT_IS_REDIS_ENABLED = False
    DEFAULT_VRF = 'default'
    DEFAULT_AGENT_PORT = 8981
    DEFAULT_AGENT_URL = '0.0.0.0'
    DEFAULT_NETQ_STREAM_ADDRESS = '127.0.0.1'
    DEFAULT_NETQ_STREAM_PORT = 7680
    DEFAULT_IS_OPTA_ENABLED = True
    DEFAULT_IS_GNMI_DISABLED = False
    DEFAULT_ENABLE_OPTA_DISCOVERY = False
    DEFAULT_OPTA_PORT = 31980
    DEFAULT_GATEWAY_PORT = 32708
    DEFAULT_HTTPS_PORT = 443
    DEFAULT_DISCOVERY_SERVER_PORT = 31980
    DEFAULT_OPID = 0
    DEFAULT_OPID_KEY = 'EhVuZXRxLWVuZHBvaW50LWdhdGV3YXkYsagDIiw3T2sweW9kR3Y4Wk9sTHU3MkwrQTRjNkhhQkU3bVpBNVlZVjEvWWgyZGJBPQ=='
    DEFAULT_SERVER = '127.0.0.1'
    DEFAULT_HIDE_VALUES = True
    BILLION_SEC_IN_MILLISEC = 1000000000000
    DEFAULT_FULLSTATE_PERIOD = 60 * 60 * 24
    DEFAULT_IS_LIBNL_ENABLED = True
    DEFAULT_IS_WJH_ENABLED = True
    DEFAULT_PROXY_PORT = 80
    COL_LEN_64_BIT = 20
    DEFAULT_ERROR_MSG = 'Failed to process command. Check /var/log/netqd.log for more details.'
    DEFAULT_CPU_LIMIT = '100'

    def __init__(self):
        raise NotImplementedError

class WjhConfKw(object):
    WJH_DROP_FILTER = 'wjh-drop-filter'
    WJH_FILTER = 'wjh_filter'
    DROP_REASONS = 'drop-reasons'
    SEVERITY = 'severity'
    L1 = 'l1'
    L2 = 'l2'
    ROUTER = 'router'
    TUNNEL = 'tunnel'
    ACL = 'acl'
    BUFFER = 'buffer'

    DROP_TYPE_MAPPING = {L1: 'Mellanox WJH L1 drops',
                         L2: 'Mellanox WJH L2 drops',
                         ROUTER: 'Mellanox WJH Router drops',
                         TUNNEL: 'Mellanox WJH Tunnel drops',
                         ACL: 'Mellanox WJH ACL drops',
                         BUFFER: 'Mellanox WJH Buffer drops'}

    DROP_REASONS_MAPPING = {
        L1: dict(PORT_ADMIN_DOWN='Port admin down',
                 AUTO_NEG_FAILURE='Auto-negotiation failure',
                 LINK_TRAINING_FAILURE='Link training failure',
                 PEER_IS_SENDING_REMOTE_FAULTS='Peer is sending remote faults',
                 BAD_SIGNAL_INTEGRITY='Bad signal integrity',
                 CABLE_IS_NOT_SUPPORTED='Cable/transceiver is not supported',
                 CABLE_IS_UNPLUGGED='Cable/transceiver is unplugged',
                 CALIBRATION_FAILURE='Calibration failure',
                 OTHER_REASON='Other reason'),

        L2: dict(MLAG_PORT_ISOLATION= 'MLAG port isolation',
                 DEST_MAC_IS_RESERVED= 'Destination MAC is reserved (DMAC=01-80-C2-00-00-0x)',
                 VLAN_TAGGING_MISMATCH= 'VLAN tagging mismatch',
                 INGRESS_VLAN_FILTERING= 'Ingress VLAN filtering',
                 INGRESS_SPANNING_TREE_FILTER= 'Ingress spanning tree filter',
                 UNICAST_MAC_TABLE_ACTION_DISCARD= 'Unicast MAC table action discard',
                 MULTICAST_EGRESS_PORT_LIST_IS_EMPTY= 'Multicast egress port list is empty',
                 PORT_LOOPBACK_FILTER= 'Port loopback filter',
                 SOURCE_MAC_IS_MULTICAST= 'Source MAC is multicast',
                 SOURCE_MAC_EQUALS_DEST_MAC= 'Source MAC equals destination MAC'),

        ROUTER: dict(NON_ROUTABLE_PCK= 'Non-routable packet',
                     BLACKHOLE_ROUTE= 'Blackhole route',
                     UNRESOLVED_NEIGHBOR = 'Unresolved neighbor/next-hop',
                     BLACKHOLE_ARP = 'Blackhole ARP/neighbor',
                     IPV6_DEST_IN_MULTICAST_SCOPE_FFX0= 'IPv6 destination in multicast scope FFx0:/16',
                     IPV6_DEST_IN_MULTICAST_SCOPE_FFX1= 'IPv6 destination in multicast scope FFx1:/16',
                     NON_IP_PACKET= 'Non IP packet',
                     UNICAST_DEST_IP_BUT_MULTICAST_DEST_MAC= 'Unicast destination IP but multicast destination MAC',
                     DEST_IP_IS_LOOPBACK_ADDR= 'Destination IP is loopback address',
                     SRC_IP_IS_MULTICAST= 'Source IP is multicast',
                     SRC_IP_IS_IN_CLASS_E= 'Source IP is in class E',
                     SRC_IP_IS_LOOPBACK_ADDR= 'Source IP is loopback address',
                     SRC_IP_IS_UNSPECIFIED= 'Source IP is unspecified',
                     CHECKSUM_OR_IPVER_OR_IPV4_IHL_TOO_SHORT= 'Checksum or IPver or IPv4 IHL too short',
                     MULTICAST_MAC_MISMATCH= 'Multicast MAC mismatch',
                     SRC_IP_EQUALS_DEST_IP= 'Source IP equals destination IP',
                     IPV4_SRC_IP_IS_LIMITED_BROADCAST= 'IPv4 source IP is limited broadcast',
                     IPV4_DEST_IP_IS_LOCAL_NETWORK= 'IPv4 destination IP is local network (destination=0.0.0.0/8)',
                     IPV4_DEST_IP_IS_LINK_LOCAL= 'IPv4 destination IP is link local',
                     INGRESS_ROUTER_INTF_IS_DISABLED= 'Ingress router interface is disabled',
                     EGRESS_ROUTER_INTF_IS_DISABLED= 'Egress router interface is disabled',
                     IPV4_ROUTING_TABLE_UNICAST_MISS= 'IPv4 routing table (LPM) unicast miss',
                     IPV6_ROUTING_TABLE_UNICAST_MISS= 'IPv6 routing table (LPM) unicast miss',
                     ROUTER_INTERFACE_LOOPBACK= 'Router interface loopback',
                     PACKET_SIZE_IS_LARGER_THAN_ROUTER_INTF_MTU= 'Packet size is larger than router interface MTU',
                     TTL_VALUE_IS_TOO_SMALL= 'TTL value is too small'),

        TUNNEL: dict(OVERLAY_SWITCH_SRC_MAC_IS_MULTICAST= 'Overlay switch - Source MAC is multicast',
                     OVERLAY_SWITCH_SRC_MAC_EQUALS_DEST_MAC= 'Overlay switch - Source MAC equals destination MAC',
                     DECAPSULATION_ERROR= 'Decapsulation error'),

        ACL: dict(INGRESS_PORT_ACL= 'Ingress port ACL',
                  INGRESS_ROUTER_ACL= 'Ingress router ACL',
                  EGRESS_ROUTER_ACL= 'Egress router ACL',
                  EGRESS_PORT_ACL= 'Egress port ACL'),

        BUFFER: dict(TAIL_DROP= 'Tail drop',
                     WRED= 'WRED',
                     PORT_TC_CONGESTION_THRESHOLD_CROSSED= 'Port TC Congestion Threshold Crossed',
                     PACKET_LATENCY_THRESHOLD_CROSSED= 'Packet Latency Threshold Crossed')
    }

class DiffState(object):

    ADDED = 0
    MODIFIED = 1
    DELETED = 2

    MAP = {
        ADDED: 'Addded',
        MODIFIED: 'Modified',
        DELETED: 'Deleted'
    }

    def __init__(self):
        raise NotImplementedError


class Heartbeat(object):
    """ These are shared across netq-agent and netq
    """
    LIVELINESS_RATE = 30
    HEARTBEAT_DEATH = 3 * LIVELINESS_RATE

    def __init__(self):
        raise NotImplementedError


class InterfaceState(object):
    """ Interface state
    """
    UP = 'up'
    DOWN = 'down'

    def __init__(self):
        raise NotImplementedError


class InterfaceDownReason(object):
    '''Reasons why an interface is down'''

    PROTODOWN = 'protodown'
    NO_CARRIER = 'no_carrier'
    DORMANT = 'dormant'
    NONE = ''

    def __init__(self):
        raise NotImplementedError


class LinkType(object):
    """Link types such as vxlan, bridge etc.
    """
    BCM = 'bcm_knet'
    MLX = 'mlx_sx_netdev'
    BOND = 'bond'
    BRIDGE = 'bridge'
    ETH = 'eth'
    IGNORE = 'ignore'
    LOOPBACK = 'loopback'
    MACVLAN = 'macvlan'
    SWP = 'swp'
    TUN = 'tun'
    VLAN = 'vlan'
    VXLAN = 'vxlan'
    VRF = 'vrf'
    IPIP = 'ipip'


class NeighborState(object):
    """ IP Neighbor state
    """
    UPDATE = 1
    DELETE = 2
    IGNORE = 3

    def __init__(self):
        raise NotImplementedError


class NetDevFmt(object):

    IFACE = 0
    RX_BYTES = 1
    RX_PKTS = 2
    RX_ERR = 3
    RX_DRP = 4
    RX_FIFO = 5
    RX_FRAME = 6
    RX_COMP = 7
    RX_MCAST = 8
    TX_BYTES = 9
    TX_PKTS = 10
    TX_ERR = 11
    TX_DRP = 12
    TX_FIFO = 13
    TX_COLL = 14
    TX_CARR = 15
    TX_COMP = 16

    def __init__(self):
        raise NotImplementedError


class Platform(object):
    """ Platform
    """
    CUMULUS = 'cumulus'
    LINUX = 'linux'
    UNKNOWN = 'UNKNOWN'
    VX = 'vx'
    CL = 'Cumulus Linux'
    MELLANOX = 'mlnx'
    SONIC = 'sonic'


class Protocol(object):
    """ Protocols
    """
    KERNEL = 'kernel'
    ZEBRA = 'zebra'
    STATIC = 'static'
    BOOT = 'boot'
    BGP = 'bgp'
    OSPF = 'ospf'
    ISIS = 'is-is'

    def __init__(self):
        raise NotImplementedError


class RedisKeyType(object):
    """Redis Key Types
    """
    HASH = 'hash'
    LIST = 'list'
    SET = 'set'
    ZSET = 'zset'
    STRING = 'string'

    def __init__(self):
        raise NotImplementedError


class RunningService(object):
    """Service name
    """
    ACLINIT = 'aclinit'
    ACLTOOL = 'acltool'
    ARP_REFRESH = 'arp_refresh'
    CLAGD = 'clagd'
    DOCKER = 'docker'
    LEDMGRD = 'ledmgrd'
    LLDPD = 'lldpd'
    MSTPD = 'mstpd'
    NTP = 'ntp'
    POED = 'poed'
    PORTWD = 'portwd'
    PTMD = 'ptmd'
    PWMD = 'pwmd'
    RSYSLOG = 'rsyslog'
    SMOND = 'smond'
    SSH = 'ssh'
    SWITCHD = 'switchd'
    QUAGGA = 'quagga'
    FRR = 'frr'
    WATCHDOG = 'wd_keepalive'
    MLX_SDK = 'sx_sdk'
    NEIGHMGRD = 'neighmgrd'
    NETQD = 'netqd'
    NETQAGENT = 'netq-agent'
    NETQ_GNMI = 'netq-gnmi'
    CTS_KUBECTL_CONFIG_CLOUD = 'cts-kubectl-config-cloud'
    CTS_KUBECTL_CONFIG = 'cts-kubectl-config'
    NETQNOTIFIER = 'netq-notifier'
    NETQSTATSPUSHD = 'netq-stats-pushd'
    CHASSISD = 'cumulus-chassisd'
    CHASSIS_SSH = 'cumulus-chassis-ssh'
    CHRONY = 'chrony'


class CentosRunningService(object):
    """Centos Specific Service name
    """
    NTP = 'ntpd'
    CHRONY = 'chronyd'


class RhelRunningService(object):
    """RHEL Specific Service name
    """
    NTP = 'ntpd'
    CHRONY = 'chronyd'


class SonicRunningService(object):
    """Sonic Specific Service name
    """
    MGMT_FWK = 'mgmt-framework'
    TELEMETRY = 'telemetry'
    SNMP = 'snmp'
    RADV = 'radv'
    DHCP_RELAY = 'dhcp_relay'
    PMON = 'pmon'
    LLDP = 'lldp'
    SYNCD = 'syncd'
    TEAMD = 'teamd'
    SWSS = 'swss'
    BGP = 'bgp'
    DATABASE = 'database'


class ServiceStatus(object):
    '''Service status. Ensure string below is updated if this changes'''
    OK = 0
    WARNING = 1
    ERROR = 2
    FAIL = 3
    NA = 4


class ServiceStatusString(object):
    '''Service status string. Ensure this stays in sync with values'''
    OK = 'ok'
    WARNING = 'warning'
    ERROR = 'error'
    FAIL = 'fail'
    NA = 'n/a'


class RouteType(object):
    """Netlink route types imported from nlmanager
    """
    # Route types
    # /usr/include/linux/rtnetlink.h
    RTN_UNSPEC = 0x00  # Unknown broker.
    RTN_UNICAST = 0x01  # A gateway or direct broker.
    RTN_LOCAL = 0x02  # A local interface broker.
    RTN_BROADCAST = 0x03  # A local broadcast route(sent as a broadcast).
    RTN_ANYCAST = 0x04  # An anycast broker.
    RTN_MULTICAST = 0x05  # A multicast broker.
    RTN_BLACKHOLE = 0x06  # A silent packet dropping broker.
    RTN_UNREACHABLE = 0x07  # Packets dropped & host unreachable ICMPs sent
    RTN_PROHIBIT = 0x08  # Packets dropped and comm prohibited ICMPs sent
    RTN_THROW = 0x09  # continue lookup in next routing table as per ip rules
    RTN_NAT = 0x0A  # A network address translation rule.
    RTN_XRESOLVE = 0x0B  # Refer to an external resolver(not implemented).


class RouteTable(object):
    """Well known route table ids"""
    DEFAULT = 254
    LOCAL = 255
    RDNBR = 10


class StpPortState(object):
    '''Spanning tree port states as reported by netlink'''
    DISABLED = 0
    LISTENING = 1
    LEARNING = 2
    FORWARDING = 3
    BLOCKING = 4
    UNINIT = 5


class ModelConstants(object):
    WJH_DROP_AGGREGATE = 'WjhDropAggregate'
    WJH_L1_DROP_AGGREGATE = 'WjhL1DropAggregate'


class DefaultBoolean(object):
    pass


class DefaultJson(object):
    pass


class DefaultString(object):
    pass


class DefaultFloat(object):
    pass


class DefaultInteger(object):
    pass
