# Copyright 2019 Cumulus Networks, Inc.  All Rights reserved.
#
# This software is subject to the Cumulus Networks End User License Agreement
# available at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt

import requests
import json


class GetRequestError(requests.exceptions.RequestException):
    def __init__(self, url, headers, response=None):
        self.method = 'GET'
        self.url = url
        self.headers = headers
        self.response = response

    def __str__(self):
        if self.response:
            return '{} request to {} -H {} failed: {}'.format(self.method, self.url, str(self.headers),
                                                              json.dumps(self.response))
        else:
            return '{} request to {} -H {} failed'.format(self.method, self.url, str(self.headers))


class PostRequestError(requests.exceptions.RequestException):
    def __init__(self, url, headers, data, response=None):
        self.method = 'POST'
        self.url = url
        self.headers = headers
        self.data = data
        self.response = response

    def __str__(self):
        if self.response:
            return '{} request to {} -H {} -d {} failed: {}'.format(self.method, self.url,
                                                                    str(self.headers), self.data,
                                                                    json.dumps(self.response))
        else:
            return '{} request to {} -H {} -d {} failed'.format(self.method, self.url, str(self.headers), self.data)


class PutRequestError(requests.exceptions.RequestException):
    def __init__(self, url, headers, data, response=None):
        self.method = 'PUT'
        self.url = url
        self.headers = headers
        self.data = data
        self.response = response

    def __str__(self):
        if self.response:
            return '{} request to {} -H {} -d {} failed: {}'.format(self.method, self.url,
                                                                    str(self.headers), self.data,
                                                                    json.dumps(self.response))
        else:
            return '{} request to {} -H {} -d {} failed'.format(self.method, self.url, str(self.headers), self.data)


class BadStatusCode(requests.exceptions.RequestException):
    def __init__(self, response):
        self.response = response

    def __str__(self):
        if isinstance(self.response, dict) and self.response.get('message'):
            return self.response.get('message')
        else:
            return str(self.response)

