# Copyright 2017 Cumulus Networks, Inc.  All Rights reserved.
#
# This software is subject to the Cumulus Networks End User License Agreement
# available at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt

import time

from netq_lib.orm import objects, HOSTNAME
from netq_lib.orm.memdb.db import AvlTree
from netq_lib.orm.memdb.query import MemQuery


class MemModel(objects.Model):
    query_cls = MemQuery
    active = objects.Boolean(json_key='active')
    hostname = objects.String(json_key='hostname')
    timestamp = objects.Float(json_key='timestamp')
    deleted = objects.Boolean(json_key='deleted')

    def __init__(self):
        self.active = True
        self.deleted = False
        self.hostname = HOSTNAME
        self.timestamp = time.time()
        super(MemModel, self).__init__()

    @classmethod
    def delete(cls, key, score):
        if key in cls.query.map:
            cls.query.map[key].remove(float(score))
            if cls.query.map[key].empty():
                cls.query.map.pop(key)

    def delete_object(self):
        cls = self.__class__
        key = cls.obj_to_key(self.__dict__)
        score = cls.obj_to_score(self.__dict__)
        cls.delete(key, score)

    @classmethod
    def flush(cls):
        cls.query.map.clear()

    @classmethod
    def save(cls, key, score, value):
        if key not in cls.query.map:
            cls.query.map[key] = AvlTree()
        cls.query.map[key].insert(float(score), value)

    def save_object(self):
        cls = self.__class__
        key = cls.obj_to_key(self.__dict__)
        score = cls.obj_to_score(self.__dict__)
        value = cls.obj_to_val(self.__dict__)
        cls.save(key, score, value)


