#!/usr/bin/env python

"""
Copyright (C) 2017 Cumulus Networks, Inc. all rights reserved

This software is subject to the Cumulus Networks End User License Agreement available
at the following locations:

Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
Cumulus Linux systems: /usr/share/cumulus/EULA.txt
"""

from .__init__ import NetworkDocopt
import unittest


class TestNetworkDocopt(unittest.TestCase):

    def test_ospf_syntax_ambiguity(self):
        """
        Avoid a problem wherein "ospf" is interpretted as a partial keyword
        match for "ospf6".  See CM-15214.
        """

        docstring = """
Usage:
    net add ospf router-id <ipv4>
    net add ospf6 interface <interface> area <ipv4>
"""

        nd = NetworkDocopt(docstring)

        # This should not match any command.
        nd.run('net add ospf interface swp13 area 23.5.1.1'.split())
        error_message = ' '.join(map(str, nd.match)) if (nd.match is not None) else ''
        self.assertIsNone(nd.match, "'{0}' is not None.".format(error_message))

        # This should match a specific command.
        command = "net add ospf6 interface swp13 area 23.5.1.1"
        nd.run(command.split())
        self.assertIsNotNone(nd.match)
        self.assertFalse(self.args['ospf'])
        self.assertTrue(self.args['ospf6'])
        self.assertEqual(' '.join(nd.match), command)


if __name__ == '__main__':
    unittest.main()
